-- authors: oreolek, grigfather, x-demon and really a lot of others who had the courage to configure awesome

-- {{{ Load libraries
-- Standard awesome library
require("awful")
require("awful.autofocus")
require("awful.rules")
-- Theme handling library
require("beautiful")
-- Notification library
require("naughty")
require("vicious")
require("lib/shifty")
-- Menu library
require("lib/freedesktop.utils")
require("lib/freedesktop.menu")
require("lib/freedesktop.desktop")
-- }}}

-- {{{ Variable definitions
-- Home variable
local home   = os.getenv("HOME")
-- Themes define colours, icons, and wallpapers
beautiful.init(home .. "/.config/awesome/themes/zenburn.lua")

-- This is used later as the default terminal and editor to run.
local terminal = "xfce4-terminal"
local editor = "xfwrite"
local editor_cmd = editor

-- Default modkey.
-- Usually, Mod4 is the key with a logo between Control and Alt.
-- If you do not like this or do not have such a key,
-- I suggest you to remap Mod4 to another key using xmodmap or other tools.
-- However, you can use another modifier like Mod1, but it may interact with others.
local modkey = "Mod4"

-- Table of layouts to cover with awful.layout.inc, order matters.
layouts ={
awful.layout.suit.tile,
awful.layout.suit.tile.left,
awful.layout.suit.tile.bottom,
awful.layout.suit.tile.top,
awful.layout.suit.fair,
awful.layout.suit.fair.horizontal,
awful.layout.suit.max,
awful.layout.suit.max.fullscreen,
awful.layout.suit.magnifier,
awful.layout.suit.floating
}

-- some commands
local commands = {}
commands.suspend = "sudo s2ram --force"
commands.lock = "xscreensaver-command --lock"
commands.screenshot = "scrot -e `mv $f ~/tmp/`"
--audio stuff
commands.raisevol = "amixer set PCM 2%+"
commands.lowervol = "amixer set PCM 2%-"
commands.mute = "amixer set PCM toggle"
commands.musnext = "mpc next"
commands.musprev = "mpc prev"
commands.muspause = "mpc pause"
commands.mustoggle = "mpc toggle"
-- other apps
commands.screensaver = "xscreensaver-command"
commands.fileman = "pcmanfm"
commands.calc = "xcalc"
commands.browser = "firefox-with-video"
-- }}}

--{{{ Tags
--{{{ Shifty: configured tags
shifty.config.tags = {
["im"] =      { layout = awful.layout.suit.tile,         mwfact=0.60, exclusive = false, solitary = false, position = 1, init = true, screen = 1, slave = true, persist = true } ,
["www"] =    { layout = awful.layout.suit.tile.bottom,   mwfact=0.30, exclusive = false , solitary = false , position = 2 } ,
["media"] =     { layout = awful.layout.suit.tile,         mwfact=0.85, exclusive = false , solitary = false , position = 3 } ,
["gimp"] =     { layout = awful.layout.suit.tile,         mwfact=0.85, exclusive = false , solitary = false , position = 4 } ,
["tixati"] =  { layout = awful.layout.suit.float,                     exclusive = false, solitary = false, position = 5 } ,
["geany"] = { layout = awful.layout.suit.tile, position = 6 } ,
["games"] =   { layout = awful.layout.suit.tile.bottom,  mwfact=0.65, exclusive = true , solitary = true , position = 7 } ,
["8"] =  { layout = awful.layout.suit.tile.bottom,  mwfact=0.65, exclusive = true , solitary = true , position = 8 } ,
["9"] =    { layout = awful.layout.suit.tile.bottom,  mwfact=0.65, exclusive = true , solitary = true , position = 9 } ,
}
--}}}

--{{{ Shifty: application matching rules
-- order here matters, early rules will be applied first
shifty.config.apps = {
{ match = { "Chromium","Google Chrome","Opera","Firefox","Namoroka","Iceweasel","Minefield"} , tag = "www"} ,
{ match = { "qutim","xchat","skype","psi","empathy"} , tag = "im"} ,
{ match = { "Mplayer.*","digikam", "easytag","rhythmbox","gmusibrowser"} , tag = "media", nopopup = true, } ,
{ match = { "gimp" } , tag = "gimp", nopopup = true, } ,
{ match = { "MPlayer", "Gnuplot", "kcalc", "gimp"                 } , float = true   } ,
{ match = { "tixati" } , tag = "tixati"  } ,
{ match = { "geany" }, tag = "geany" },
{ match = { "Braid","WoW", "Warcraft", "Neverwinter", "nwn" }, tag = "games" },
{ match = { terminal } , honorsizehints = false, slave = true   } ,
-- match all
{ match = { "" } ,  buttons = awful.util.table.join(
awful.button({ }, 1, function (c) client.focus = c; c:raise() end),
awful.button({ modkey }, 1, awful.mouse.client.move),
awful.button({ modkey }, 3, awful.mouse.client.resize))
}
}
--}}}

--{{{ Shifty: default tag creation rules
-- parameter description
--  * floatBars : if floating clients should always have a titlebar
--  * guess_name : wether shifty should try and guess tag names when creating new (unconfigured) tags
--  * guess_position: as above, but for position parameter
--  * run : function to exec when shifty creates a new tag
--  * remember_index: ?
--  * all other parameters (e.g. layout, mwfact) follow awesome's tag API
shifty.config.defaults = {
ncol = 1,
mwfact = 0.60,
floatBars=true,
guess_name=true,
run = function(tag)
naughty.notify({ text = "Shifty Created #"..
(awful.tag.getproperty(tag,"position") or shifty.tag2index(mouse.screen,tag))..": "..
(tag.name or "foo")
})
end,
guess_position = true,
remember_index = true
}
--}}}
--}}}

-- {{{ Menu
-- applications menu
freedesktop.utils.terminal = terminal

menu_items = freedesktop.menu.new()
myawesomemenu = {
{ "manual", terminal .. " -e man awesome", freedesktop.utils.lookup_icon({ icon = 'help' }) },
{ "edit config", editor_cmd .. " " .. awful.util.getdir("config") .. "/rc.lua", freedesktop.utils.lookup_icon({ icon = 'package_settings' }) },
{ "restart", awesome.restart, freedesktop.utils.lookup_icon({ icon = '' }) },
{ "quit", awesome.quit, freedesktop.utils.lookup_icon({ icon = '' }) }
}
mycommons = {
{ "empathy", "empathy" },
{ "firefox", "firefox" },
{ "skype", "skype" },
{ "xchat", "xchat" },
{ "pcmanfm", "pcmanfm" }
}
table.insert(menu_items, { "awesome", myawesomemenu, beautiful.awesome_icon })
table.insert(menu_items, { "Common Apps", mycommons, beautiful.awesome_icon })
table.insert(menu_items, { "open terminal", terminal, freedesktop.utils.lookup_icon({icon = 'terminal'}) })

mymainmenu = awful.menu.new({ items = menu_items })

-- }}}

mytasklist = {}
mytasklist.buttons = awful.util.table.join(
                     awful.button({ }, 1, function (c)
                                              if not c:isvisible() then
                                                  awful.tag.viewonly(c:tags()[1])
                                              end
                                              client.focus = c
                                              c:raise()
                                          end),
                     awful.button({ }, 3, function ()
                                              if instance then
                                                  instance:hide()
                                                  instance = nil
                                              else
                                                  instance = awful.menu.clients({ width=250 })
                                              end
                                          end),
                     awful.button({ }, 4, function ()
                                              awful.client.focus.byidx(1)
                                              if client.focus then client.focus:raise() end
                                          end),
                     awful.button({ }, 5, function ()
                                              awful.client.focus.byidx(-1)
                                              if client.focus then client.focus:raise() end
                                          end))

--naughty.notify{text="rc.lua, line 191", timeout = 0}

-- {{{ Wibox
-- {{{ Widgets configuration
--
-- {{{ Reusable separators
spacer    = widget({ type = "textbox"  })
separator = widget({ type = "imagebox" })
spacer.text     = " "
separator.image = image(beautiful.widget_sep)
-- }}}

mylauncher = awful.widget.launcher({ image = beautiful.awesome_icon, menu = mymainmenu })

-- {{{ CPU usage
cpuicon = widget({ type = "imagebox" })
cpuicon.image = image(beautiful.widget_cpu)
-- Initialize widgets
cpugraph  = awful.widget.graph()
-- Graph properties
cpugraph:set_width(40)
cpugraph:set_height(14)
cpugraph:set_background_color(beautiful.fg_off_widget)
cpugraph:set_gradient_angle(0)
cpugraph:set_gradient_colors({ beautiful.fg_end_widget,
beautiful.fg_center_widget, beautiful.fg_widget
}) -- Register widgets
vicious.register(cpugraph, vicious.widgets.cpu, "$1")
-- }}}

-- {{{ Memory usage
memicon = widget({ type = "imagebox" })
memicon.image = image(beautiful.widget_mem)
-- Initialize widget
membar = awful.widget.progressbar()
-- Pogressbar properties
membar:set_width(10)
membar:set_height(12)
membar:set_vertical(true)
membar:set_background_color(beautiful.fg_off_widget)
membar:set_border_color(beautiful.border_widget)
membar:set_gradient_colors({ beautiful.fg_widget,
beautiful.fg_center_widget, beautiful.fg_end_widget
}) -- Register widget
vicious.register(membar, vicious.widgets.mem, "$1", 13)
-- }}}

-- {{{ Date and time
-- dateicon = widget({ type = "imagebox" })
-- Initialize widget
datewidget = widget({ type = "textbox" })
-- Register widget
vicious.register(datewidget, vicious.widgets.date, "%d %b, %R", 60)
-- }}}

-- {{{ System tray
systray = widget({ type = "systray" })
-- }}}
-- }}}

-- {{{ Wibox initialisation
wibox     = {}
promptbox = {}
layoutbox = {}
taglist   = {}
taglist.buttons = awful.util.table.join(
awful.button({ }, 1, awful.tag.viewonly),
awful.button({ modkey }, 1, awful.client.movetotag),
awful.button({ }, 3, awful.tag.viewtoggle),
awful.button({ modkey }, 3, awful.client.toggletag),
awful.button({ }, 4, awful.tag.viewnext),
awful.button({ }, 5, awful.tag.viewprev
))
for s = 1, screen.count() do
-- Create a promptbox
promptbox[s] = awful.widget.prompt({ layout = awful.widget.layout.horizontal.leftright })
-- Create a layoutbox
layoutbox[s] = awful.widget.layoutbox(s)
layoutbox[s]:buttons(awful.util.table.join(
awful.button({ }, 1, function () awful.layout.inc(layouts, 1)  end),
awful.button({ }, 3, function () awful.layout.inc(layouts, -1) end),
awful.button({ }, 4, function () awful.layout.inc(layouts, 1)  end),
awful.button({ }, 5, function () awful.layout.inc(layouts, -1) end)
))
-- Create a taglist
taglist[s] = awful.widget.taglist.new(s, awful.widget.taglist.label.all, taglist.buttons)
-- Create a tasklist widget
mytasklist[s] = awful.widget.tasklist(function(c)
                                        return awful.widget.tasklist.label.currenttags(c, s)
                                      end, mytasklist.buttons)
-- Create the wibox
wibox[s] = awful.wibox({      screen = s,
fg = beautiful.fg_normal,
bg = beautiful.bg_normal, position = "top",
border_color = beautiful.border_focus,
border_width = beautiful.border_width
})
-- Add widgets to the wibox
wibox[s].widgets = {
{taglist[s],
mylauncher,
layoutbox[s],
promptbox[s],
["layout"] = awful.widget.layout.horizontal.leftright
},
s == screen.count() and systray or nil,
separator, datewidget,
separator, membar.widget, memicon,
separator, cpugraph.widget, cpuicon,
separator, mytasklist[s],
separator, ["layout"] = awful.widget.layout.horizontal.rightleft
}
end

-- {{{ Shifty: initialize shifty
-- the assignment of shifty.taglist must always be after its actually initialized
-- with awful.widget.taglist.new()
shifty.taglist = taglist
-- }}}

-- }}}
-- }}}

-- {{{ Mouse bindings
root.buttons(awful.util.table.join(
awful.button({ }, 3, function () mymainmenu:toggle() end),
awful.button({ }, 4, awful.tag.viewnext),
awful.button({ }, 5, awful.tag.viewprev)
))
-- }}}

-- {{{ Key bindings
globalkeys = awful.util.table.join(
--user defined
awful.key({                   }, "XF86PowerOff",         function() awful.util.spawn_with_shell(commands.suspend) end ),
awful.key({ modkey,           }, "F12",                  function () awful.util.spawn_with_shell(commands.lock) end),
--audio stuff
awful.key({                   }, "XF86AudioMute",        function() awful.util.spawn_with_shell(commands.mute) end ),
awful.key({                   }, "XF86AudioRaiseVolume", function() awful.util.spawn_with_shell(commands.raisevol) end ),
awful.key({                   }, "XF86AudioLowerVolume", function() awful.util.spawn_with_shell(commands.lowervol) end ),
awful.key({                   }, "XF86AudioNext",        function() awful.util.spawn_with_shell(commands.musnext) end ),
awful.key({                   }, "XF86AudioPrev",        function() awful.util.spawn_with_shell(commands.musprev) end ),
awful.key({                   }, "XF86AudioPlay",        function() awful.util.spawn_with_shell(commands.mustoggle) end ),
awful.key({                   }, "XF86Tools",            function() awful.util.spawn_with_shell(commands.musplay) end ),
awful.key({                   }, "XF86Calculator",       function() awful.util.spawn_with_shell(commands.calc) end ),
awful.key({                   }, "XF86ScreenSaver",      function() awful.util.spawn_with_shell(commands.screensaver) end ),
awful.key({                   }, "Print",                function() awful.util.spawn_with_shell(commands.screenshot) end ),
--default bindings
awful.key({ modkey,           }, "Left",   awful.tag.viewprev       ),
awful.key({ modkey,           }, "Right",  awful.tag.viewnext       ),
awful.key({ modkey, "Control" }, "Left",   shifty.shift_prev        ),
awful.key({ modkey, "Control" }, "Right",  shifty.shift_next       ),
awful.key({ modkey            }, "t",      function() shifty.add({ rel_index = 1 }) end),
awful.key({ modkey, "Control" }, "t",      function() shifty.add({ rel_index = 1, nopopup = true }) end),
--awful.key({ modkey            }, "r",      shifty.rename), -- //TODO: fix renaming
awful.key({ modkey            }, "w",      shifty.del),
awful.key({ modkey,           }, "u",      awful.client.urgent.jumpto),
awful.key({ modkey,           }, "n",
function ()
awful.client.focus.byidx( 1)
if client.focus then client.focus:raise() end
end),
awful.key({ modkey,           }, "k",
function ()
awful.client.focus.byidx(-1)
if client.focus then client.focus:raise() end
end),
awful.key({ modkey,           }, "Tab",
function ()
awful.client.focus.history.previous()
if client.focus then
client.focus:raise()
end
end),

-- Standard program
awful.key({ modkey,           }, "Return", function () awful.util.spawn(terminal) end),
awful.key({ modkey, "Control" }, "r", awesome.restart),
awful.key({ modkey, "Shift"   }, "q", awesome.quit),
-- Layout manipulation
awful.key({ modkey,           }, "l",     function () awful.tag.incmwfact( 0.05)    end),
awful.key({ modkey,           }, "h",     function () awful.tag.incmwfact(-0.05)    end),
awful.key({ modkey, "Shift"   }, "h",     function () awful.tag.incnmaster( 1)      end),
awful.key({ modkey, "Shift"   }, "l",     function () awful.tag.incnmaster(-1)      end),
awful.key({ modkey, "Control" }, "h",     function () awful.tag.incncol( 1)         end),
awful.key({ modkey, "Control" }, "l",     function () awful.tag.incncol(-1)         end),
awful.key({ modkey,           }, "space", function () awful.layout.inc(layouts,  1) end),
awful.key({ modkey, "Shift"   }, "space", function () awful.layout.inc(layouts, -1) end),
awful.key({ modkey },            "r",     function () promptbox[mouse.screen]:run() end)
)
clientkeys = awful.util.table.join(
awful.key({ modkey,           }, "f",      function (c) c.fullscreen = not c.fullscreen  end),
awful.key({ modkey, "Shift"   }, "c",      function (c) c:kill()                         end),
awful.key({ modkey, "Control" }, "space",  awful.client.floating.toggle                     ),
awful.key({ modkey, "Control" }, "Return", function (c) c:swap(awful.client.getmaster()) end),
awful.key({ modkey,           }, "o",      awful.client.movetoscreen                        ),
awful.key({ modkey, "Shift"   }, "r",      function (c) c:redraw()                       end),
awful.key({ modkey,           }, "j",      function (c) c.minimized = not c.minimized    end),
awful.key({ modkey,           }, "m",
function (c)
c.maximized_horizontal = not c.maximized_horizontal
c.maximized_vertical   = not c.maximized_vertical
end)
)

-- {{{ bindings / global / shifty.getpos
for i=1, ( shifty.config.maxtags or 9 ) do
globalkeys = awful.util.table.join(globalkeys, awful.key({ modkey }, i,
function ()
local t = awful.tag.viewonly(shifty.getpos(i))
end))
globalkeys = awful.util.table.join(globalkeys, awful.key({ modkey, "Control" }, i,
function ()
local t = shifty.getpos(i)
t.selected = not t.selected
end))
globalkeys = awful.util.table.join(globalkeys, awful.key({ modkey, "Control", "Shift" }, i,
function ()
if client.focus then
awful.client.toggletag(shifty.getpos(i))
end
end))
-- move clients to other tags
globalkeys = awful.util.table.join(globalkeys, awful.key({ modkey, "Shift" }, i,
function ()
if client.focus then
local t = shifty.getpos(i)
awful.client.movetotag(t)
awful.tag.viewonly(t)
end
end))
end
-- }}}

clientbuttons = awful.util.table.join(
awful.button({ }, 1, function (c) client.focus = c; c:raise() end),
awful.button({ modkey }, 1, awful.mouse.client.move),
awful.button({ modkey }, 3, awful.mouse.client.resize))

-- Register keys
root.keys(globalkeys)
shifty.config.globalkeys = globalkeys
shifty.config.clientkeys = clientkeys
-- }}}

-- {{{ Signals
-- Signal function to execute when a new client appears.
client.add_signal("manage", function (c, startup)
-- Add a titlebar
-- awful.titlebar.add(c, { modkey = modkey })
-- remove gaps
c.size_hints_honor = false

-- Enable sloppy focus
c:add_signal("mouse::enter", function(c)
if awful.layout.get(c.screen) ~= awful.layout.suit.magnifier
and awful.client.focus.filter(c) then
client.focus = c
end
end)

if not startup then
-- Set the windows at the slave,
-- i.e. put it at the end of others instead of setting it master.
-- awful.client.setslave(c)

-- Put windows in a smart way, only if they does not set an initial position.
if not c.size_hints.user_position and not c.size_hints.program_position then
awful.placement.no_overlap(c)
awful.placement.no_offscreen(c)
end
end
end)
shifty.init()
client.add_signal("focus", function(c) c.border_color = beautiful.border_focus end)
client.add_signal("unfocus", function(c) c.border_color = beautiful.border_normal end)
-- }}}
